var init = (function () {
    'use strict';

    const ExtensionUtils = imports.misc.extensionUtils;
    const Me = ExtensionUtils.getCurrentExtension();
    const Gio = imports.gi.Gio;
    class FocusSettings {
        constructor(settings) {
            this.settings = settings;
        }
        get focus_opacity() {
            return this.settings.get_uint('focus-opacity');
        }
        set_focus_opacity(val) {
            this.settings.set_uint('focus-opacity', val);
        }
        get special_focus_opacity() {
            return this.settings.get_uint('special-focus-opacity');
        }
        set_special_focus_opacity(val) {
            this.settings.set_uint('special-focus-opacity', val);
        }
        get inactive_opacity() {
            return this.settings.get_uint('inactive-opacity');
        }
        set_inactive_opacity(val) {
            this.settings.set_uint('inactive-opacity', val);
        }
    }
    function get_settings() {
        var _a;
        const schema = Gio.SettingsSchemaSource.new_from_directory(Me.dir.get_child('schemas').get_path(), Gio.SettingsSchemaSource.get_default(), false);
        return new FocusSettings(new Gio.Settings({
            settings_schema: (_a = schema.lookup('org.gnome.shell.extensions.focus', true)) !== null && _a !== void 0 ? _a : undefined,
        }));
    }

    const ExtensionUtils$1 = imports.misc.extensionUtils;
    const Me$1 = ExtensionUtils$1.getCurrentExtension();
    const { byteArray } = imports;
    const GLib = imports.gi.GLib;
    const Meta = imports.gi.Meta;
    const DEFAULT_OPACITY = 255;
    let create_signal = undefined;
    let settings;
    let special_focus_list;
    let ignore_focus_list;
    const WINDOW_TYPES = [Meta.WindowType.NORMAL];
    function set_opacity(win, actor, value) {
        if (!WINDOW_TYPES.includes(win.window_type)) {
            return;
        }
        if (ignore_focus_list &&
            (ignore_focus_list.includes(win.get_wm_class()) ||
                ignore_focus_list.includes(win.get_wm_class_instance()) ||
                ignore_focus_list.includes(win.get_title()))) {
            return;
        }
        for (const child of actor.get_children()) {
            if (child.set_opacity) {
                child.set_opacity(value);
            }
        }
        if (actor.set_opacity) {
            actor.set_opacity(value);
        }
    }
    function translate_opacity(percentage) {
        return (DEFAULT_OPACITY * percentage) / 100;
    }
    function focus_changed() {
        if (!settings) {
            return;
        }
        const focus_opacity = translate_opacity(settings.focus_opacity);
        const inactive_opacity = translate_opacity(settings.inactive_opacity);
        const special_opacity = translate_opacity(settings.special_focus_opacity);
        for (const actor of global.get_window_actors()) {
            const meta_win = actor.get_meta_window();
            set_opacity(meta_win, actor, meta_win.has_focus() || meta_win.is_fullscreen()
                ? special_focus_list &&
                    (special_focus_list.includes(meta_win.get_wm_class()) ||
                        special_focus_list.includes(meta_win.get_wm_class_instance()))
                    ? special_opacity
                    : focus_opacity
                : inactive_opacity);
        }
    }
    function enable() {
        log(`enabling ${Me$1.metadata.name} version ${Me$1.metadata.version}`);
        settings = get_settings();
        create_signal = global.display.connect('window-created', function (_, win) {
            if (!WINDOW_TYPES.includes(win.window_type)) {
                return;
            }
            win._focus_extension_signal = win.connect('focus', focus_changed);
            GLib.timeout_add(GLib.PRIORITY_DEFAULT, 250, function () {
                try {
                    focus_changed();
                }
                catch (err) {
                    log(`Error on new window: ${err}`);
                }
                return false;
            });
        });
        const special_file = GLib.build_filenamev([GLib.get_user_config_dir(), Me$1.metadata.name, 'special_focus.json']);
        try {
            const special_focus_load = GLib.file_get_contents(special_file);
            if (special_focus_load[0]) {
                special_focus_list = JSON.parse(byteArray.toString(special_focus_load[1]));
            }
        }
        catch (error) {
            log(`${Me$1.metadata.name}: failed to load special focus file ${error}`);
        }
        const ignore_file = GLib.build_filenamev([GLib.get_user_config_dir(), Me$1.metadata.name, 'ignore_focus.json']);
        try {
            const ignore_focus_load = GLib.file_get_contents(ignore_file);
            if (ignore_focus_load[0]) {
                ignore_focus_list = JSON.parse(byteArray.toString(ignore_focus_load[1]));
            }
        }
        catch (error) {
            log(`${Me$1.metadata.name}: Failed to load ignore file ${error}`);
        }
        const inactive_opacity = translate_opacity(settings === null || settings === void 0 ? void 0 : settings.inactive_opacity);
        for (const actor of global.get_window_actors()) {
            const win = actor.get_meta_window();
            if (!WINDOW_TYPES.includes(win.window_type)) {
                continue;
            }
            if (!win._focus_extension_signal) {
                win._focus_extension_signal = win.connect('focus', focus_changed);
            }
            if (!win.has_focus()) {
                set_opacity(win, actor, inactive_opacity);
            }
        }
    }
    function disable() {
        log(`disabling ${Me$1.metadata.name} version ${Me$1.metadata.version}`);
        if (create_signal) {
            global.display.disconnect(create_signal);
            create_signal = undefined;
        }
        for (const actor of global.get_window_actors()) {
            const win = actor.get_meta_window();
            if (win && win._focus_extension_signal) {
                win.disconnect(win._focus_extension_signal);
                delete win._focus_extension_signal;
            }
        }
        for (const actor of global.get_window_actors()) {
            set_opacity(actor.get_meta_window(), actor, DEFAULT_OPACITY);
        }
        special_focus_list = undefined;
        ignore_focus_list = undefined;
        settings = undefined;
    }
    function extension () {
        return {
            enable,
            disable,
        };
    }

    return extension;

}());
